package com.interdataworking.gateway.sqlddl;

import java.io.*;
import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;
import com.interdataworking.*;
import com.interdataworking.mm.*;
import com.interdataworking.mm.alg.*;
import edu.stanford.db.rdf.query.*;
import edu.stanford.db.rdf.query.memory.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import edu.stanford.db.rdf.vocabulary.order_20000527.RDFX;
//import org.w3c.rdf.vocabulary.dublin_core_19990702.DC;
//import edu.stanford.db.rdf.query.orderings.*;

import com.interdataworking.gateway.xml.*;

public class SQLDDL2XSD implements UntypedGateway {

  Obj2SQLDDL os = new Obj2SQLDDL();
  XMLVocabulary XML = new XMLVocabulary();
  XSDVocabulary XSD = new XSDVocabulary();

  void add(Model result, Resource s, Resource p, RDFNode o) throws ModelException {

    result.add(result.getNodeFactory().createStatement(s, p, o));
  }

  void addChild(Resource parent, Resource child, Resource tagName, int pos, Model m) throws ModelException {

    Statement st = m.getNodeFactory().createStatement(parent, XML.child, child);
    m.add(st);
    add(m, child, XML.tagName, tagName);
    add(m, st, RDFX.order, m.getNodeFactory().createLiteral(String.valueOf(pos)));
  }

  void addAttribute(Resource node, Resource attName, String attValue, Model m) throws ModelException {

    add(m, node, attName, m.getNodeFactory().createLiteral(attValue));
  }

  void addMap(Resource src, Resource dest, List map) {

    map.add(new MapPair(src, dest, 1.0));
  }

  public void convert(Model model, Model result, List map) throws ModelException {

    //    System.err.println("Num: " + model.find(null, RDF.type, XML.TableDef).size());

    NodeIterator it = new NodeResultSet(os.Q_TBL, model);
    //    w.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n");
    //    w.write("<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n");

    // create schema node

    NodeFactory f = model.getNodeFactory();
    Resource schema = f.createUniqueResource();
    add(result, schema, RDF.type, XML.XMLDOM);
    add(result, schema, XML.usesNamespace, f.createResource(XSD._Namespace));
    add(result, schema, XML.tagName, XSD.schema);

    int schemaPos = 0; // order of comlex types in schema

    while(it.next()) {

      Resource type = f.createUniqueResource();
      addMap((Resource)it.get("T"), type, map);
      addChild(schema, type, XSD.complexType, ++schemaPos, result); // add complex type
      addAttribute(type, XSD.name, String.valueOf(it.get("N")), result); // name of complex type

      Resource sequence = f.createUniqueResource();
      addChild(type, sequence, XSD.sequence, 1, result); // add sequence tag

      // write column definition

      int elementPos = 0; // order of elements in complex type

      String description = null; // optional column description

      NodeIterator itc = new NodeResultSet(os.Q_COL, model);
      itc.set("T", it.get("T"));
      boolean first_itc = true;
      while(itc.next()) {

	if(!first_itc) {
	  //	  w.write(",");
	  if(description != null) {
	    //	    w.write("\t\t<!-- " + description + " -->");
	    description = null;
	  }
	  //	  w.write("\n");
	}
	first_itc = false;
	
	String elementType = "";

	// REFERENCES
	if(itc.get("UC") != null) {

	  // obtain table name of the referenced column

	  elementType = String.valueOf(itc.get("UCTN"));

	  /*
	  w.write(" REFERENCES " + itc.get("UCTN"));
	  RDFNode thatColName = (RDFNode)itc.get("UCN");
	  if(!itc.get("N").equals(thatColName))
	    w.write("(" + thatColName + ")");
	  */
	} else { // regular field

	  elementType = String.valueOf(itc.get("V"));

	  String e = elementType.toLowerCase();
	  if(e.startsWith("int"))
	    elementType = "xsd:int";
	  else if(e.startsWith("string") || e.startsWith("char") || e.startsWith("varchar"))
	    elementType = "xsd:string";
	  else if(e.startsWith("dec") || e.startsWith("double") || e.startsWith("num"))
	    elementType = "xsd:real";
	  else if(e.startsWith("date"))
	    elementType = "xsd:date";
	}

	Resource element = f.createUniqueResource();
	addMap((Resource)itc.get("C"), element, map);
	addChild(sequence, element, XSD.element, ++elementPos, result);
	addAttribute(element, XSD.name, String.valueOf(itc.get("N")), result);
	addAttribute(element, XSD.type, elementType, result);

	//	w.write("\t\t<xsd:element name=\"" + itc.get("N") + "\" type=\"" + elementType + "\"");

	/*
	// column definition parameters
	NodeIterator itp = new NodeResultSet(Q_PAR, model);
	itp.set("CT", itc.get("CT"));
	boolean first_itp = true;
	if(itp.next()) {
	  w.write("(");
	  do {
	    if(!first_itp)
	      w.write(',');
	    w.write("" + itp.get("P"));
	    first_itp = false;
	  } while(itp.next());
	  w.write(')');
	}
	*/

	// description
	Literal dlit = (Literal)itc.get("DESC");
	description = dlit != null ? dlit.getLabel() : null;

	// default value
	if(itc.get("D") != null)
	  addAttribute(element, XSD.default_, String.valueOf(itc.get("D")), result);
	//	  w.write(" default=\"" + itc.get("D") + "\"");
	
	// NOT NULL
	if(itc.get("CC") != null)
	  addAttribute(element, XSD.use, "required", result);
	//	  w.write(" use=\"required\"");
	//	  w.write(" NOT NULL");
	//	w.write('\n');

	//	w.write("/>\n");
      }
      
      /*
      String ksep = (description != null) ? ",\t# " + description + "\n" : ",\n";

      // WRITE KEY DEFINITIONS
      NodeIterator itk = new NodeResultSet(Q_KEY, model);
      itk.set("T", it.get("T"));
      while(itk.next()) {

	w.write(ksep);
	ksep = ",\n"; description = null;
	// determine key type
	Object ktObj = itk.get("KT");
	if(!(ktObj instanceof Resource))
	   System.err.println("Error: key type is not a resource: " + ktObj);
	Resource kt = (Resource)ktObj;
	String ktstr = "KEY";
	if(R.PrimaryKey.equals(kt))
	  ktstr = "PRIMARY KEY";
	else if(R.UniqueKey.equals(kt))
	  ktstr = "UNIQUE KEY";

	w.write("\t" + ktstr);

	String keyName = itk.get("N") != null ? itk.get("N").getLabel() : null;
	if(keyName != null)
	  w.write(" " + keyName);

	// key components
	NodeIterator itkc = new NodeResultSet(Q_KEY_PAR, model);
	itkc.set("K", itk.get("K"));
	boolean first_itkc = true;
	if(itkc.next()) {
	  w.write(" (");
	  do {
	    if(!first_itkc)
	      w.write(',');
	    w.write("" + itkc.get("N"));
	    first_itkc = false;
	  } while(itkc.next());
	  w.write(')');
	}
      }
      */

      if(description != null) {
	//	w.write(" <-- " + description + " -->\n");
	description = null;
      }

      //      w.write("\t</xsd:sequence>\n");
      //      w.write("\t</xsd:complexType>\n");
    }

    //    w.write("</xsd:schema>\n");
    //    w.flush();
  }

  public List execute(List input) throws ModelException {

    Model src = (Model)input.get(0);

    List map = new ArrayList();
    Model result = src.create();

    convert(src, result, map);

    ArrayList l = new ArrayList();
    l.add(result);
    l.add(MapPair.asModel(src, map));
    return l;
  }
  

  public int getMinInputLen() { return 1; }

  public int getMaxInputLen() { return 1; }

  public int getMinOutputLen() { return 2; }

  public int getMaxOutputLen() { return 2; }

}
