package com.interdataworking.gateway.sqlview;

import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.util.*;
import com.interdataworking.*;
import com.interdataworking.mm.alg.*;
import com.interdataworking.gateway.sqlddl.Obj2SQLDDL;
import edu.stanford.db.rdf.query.*;
import edu.stanford.db.rdf.query.memory.*;
import edu.stanford.db.rdf.query.orderings.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import edu.stanford.db.rdf.util.*;

public class GenerateView implements UntypedGateway {

  Obj2SQLDDL sqlddl = new Obj2SQLDDL();
  PowerModel model, left, right;
  int fctIdx = 0;

  public Model generateView(List map, Model leftM, Model rightM) throws ModelException {

    this.model = PowerModel.create(leftM.create());
    this.right = PowerModel.create(rightM);
    this.left = PowerModel.create(leftM);

    Map rmap = MapPair.toJavaSetMap(map, true); // right node -> set of left nodes
    Map lmap = MapPair.toJavaSetMap(map, false); // left node -> set of right nodes

    // drive view generation by the table definitions in right model

    for(NodeIterator irt = new NodeResultSet(sqlddl.Q_TBL, right); irt.next();) {

      Resource rtable = (Resource)irt.get("T");
      PowerModel tm = PowerModel.create(model.create()); // temporary view definition
      Resource view = tm.unique();
      tm.add(view, RDF.type, SQLViewVocabulary.CreateView);
      Resource cvTable = tm.unique();
      tm.add(view, SQLViewVocabulary.cvTable, cvTable);
      tm.add(cvTable, RDF.type, SQLViewVocabulary.CVTable);
      tm.add(cvTable, SQLViewVocabulary.tableRef, rtable);

      List columns = right.getOrderedList(rtable, sqlddl.R.column);

      for(int i=0; i < columns.size(); i++) {

	Resource cvField = tm.unique();
	tm.add(view, SQLViewVocabulary.cvField, cvField, i+1);
	tm.add(cvField, RDF.type, SQLViewVocabulary.CVField);
	tm.add(cvField, SQLViewVocabulary.fieldRef, (Resource)columns.get(i));
      }

      Resource select = tm.unique();
      tm.add(view, SQLViewVocabulary.as, select);
      tm.add(select, RDF.type, SQLViewVocabulary.Select);

      Map table2alias = new HashMap(); // Resource -> Resource
      List lterms = new ArrayList();
      for(Iterator irf = columns.iterator(); irf.hasNext();) {

	Resource rfield = (Resource)irf.next();
	Set lfields = (Set)rmap.get(rfield);
	lterms.add(createTerm(tm, lfields, table2alias, lmap, rtable));
      }
      tm.addOrdered(select, SQLViewVocabulary.selectTerm, lterms);

      // add FROM clause if needed
      if(table2alias.size() > 0) {
	Resource from = tm.unique();
	tm.add(from, RDF.type, SQLViewVocabulary.FromClause);
	tm.add(select, SQLViewVocabulary.fromClause, from);

	tm.addOrdered(from, SQLViewVocabulary.inFromClause, table2alias.values().iterator());
      }

      // FIXME: add WHERE clause

//       for(Iterator ia = table2alias.entrySet().iterator(); ia.hasNext();) {

// 	Map.Entry e = (Map.Entry)ia.next();
	
//       }

      model.addAll(tm);
    }

    return model;
  }

  Resource createFieldTerm(PowerModel tm, Resource field, Map table2alias) throws ModelException {

    Resource stmt = tm.unique();
    tm.add(stmt, RDF.type, SQLViewVocabulary.FieldTerm);
    tm.add(stmt, SQLViewVocabulary.fieldRef, field);

    // determine model
    Resource ltable = left.subjectR(sqlddl.R.column, field);

    Resource aliasDef = (Resource)table2alias.get(ltable);
    if(aliasDef == null) {

      aliasDef = tm.unique();
      tm.add(aliasDef, RDF.type, SQLViewVocabulary.AliasClause);
      tm.add(aliasDef, SQLViewVocabulary.tableRef, ltable);
      tm.add(aliasDef, SQLViewVocabulary.aliasName, String.valueOf((char)((int)'A' + table2alias.size())));

      table2alias.put(ltable, aliasDef);
    }

    tm.add(stmt, SQLViewVocabulary.aliasClause, aliasDef);
    return stmt;
  }

  Resource createTerm(PowerModel tm, Set lfields, Map table2alias, Map lmap, Resource rtable) throws ModelException {

    Resource stmt;

    int size = lfields == null ? 0 : lfields.size();

    switch(size) {

    case 0:
      {
	stmt = tm.unique();
	tm.add(stmt, RDF.type, SQLViewVocabulary.Constant);
	tm.add(stmt, SQLViewVocabulary.constantValue, "NULL");
	break;
      }
    case 1:
      {
	MapPair pair = (MapPair)lfields.iterator().next();
	Resource lfield = (Resource)pair.getLeft();
	stmt = createFieldTerm(tm, lfield, table2alias);

	Set rfields = (Set)lmap.get(lfield);

	if(rfields.size() > 1) {
	  // wrap it into a function since this field maps onto multiple different fields
	  // need to make sure that multiple targets are within the same create view statement!
	  // FIXME
	  int rfieldsInRTable = 0;
	  for(Iterator it = rfields.iterator(); it.hasNext();) {

	    MapPair p = (MapPair)it.next();
	    Resource rfield = (Resource)p.getRight();
	    Resource itsRTable = right.subjectR(sqlddl.R.column, rfield);
	    if(rtable.equals(itsRTable))
	      rfieldsInRTable++;
	  }

	  if(rfieldsInRTable > 1) {
	    Resource f = tm.unique();
	    tm.add(f, RDF.type, SQLViewVocabulary.FunctionalTerm);
	    tm.add(f, SQLViewVocabulary.operatorName, "fct" + (fctIdx++));
	    tm.add(f, SQLViewVocabulary.term, stmt);
	    stmt = f; // to return
	  }
	}
	break;
      }
    default:
      {
	stmt = tm.unique();
	tm.add(stmt, RDF.type, SQLViewVocabulary.FunctionalTerm);
	tm.add(stmt, SQLViewVocabulary.operatorName, "fct" + (fctIdx++));
	List terms = new ArrayList();
	for(Iterator it = lfields.iterator(); it.hasNext();) {

	  MapPair pair = (MapPair)it.next();
	  Resource lfield = (Resource)pair.getLeft();
	  terms.add(createFieldTerm(tm, lfield, table2alias));
	}
	tm.addOrdered(stmt, SQLViewVocabulary.term, terms);
	break;
      }
    }
    return stmt;
  }

  public List execute(List input) throws ModelException {

    Model l = (Model)input.get(0);
    Model left = (Model)input.get(1);
    Model right = (Model)input.get(2);

    Model m = generateView(MapPair.toMapPairs(l), left, right);

    List result = new ArrayList();
    result.add(m);
    return result;
  }

  public int getMinInputLen() { return 3; }

  public int getMaxInputLen() { return 3; }

  public int getMinOutputLen() { return 1; }

  public int getMaxOutputLen() { return 1; }
}
