package com.interdataworking.gateway.sqlview;

import java.io.*;
import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;

import com.interdataworking.*;
import com.interdataworking.mm.alg.*;

import edu.stanford.db.rdf.query.*;
import edu.stanford.db.rdf.query.memory.*;
import edu.stanford.db.rdf.query.orderings.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;

// import edu.stanford.db.rdf.util.*;

public class LanguageCallbackView extends LanguageCallbackAdapter {

  //  SQLDDLVocabulary SQL = SQLDDLVocabulary.create();
  //  Cache cache = new Cache(100, false);

  public String toString() {

    return "Language callback for SQL views";
  }

  public boolean isCallbackFor(Model m) throws ModelException {

    return m.find(null, null, SQLViewVocabulary.CreateView).size() > 0;
  }

  public Collection getConstraints() throws ModelException {

    List l = new ArrayList();
    int NONE = Merge.NONE;

    com.interdataworking.gateway.sqlview.SQLViewVocabulary VIEW = new com.interdataworking.gateway.sqlview.SQLViewVocabulary();

    l.add(new Merge.CardinalityConstraint(VIEW.cvLink, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.fromClause, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.whereClause, 1, 1));

    l.add(new Merge.CardinalityConstraint(VIEW.cvTable, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.cvField, 1, NONE));
    l.add(new Merge.CardinalityConstraint(VIEW.whenTerm, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.thenTerm, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.headTerm, 1, 1));

    Set terms = new HashSet();
    terms.add(VIEW.term);
    terms.add(VIEW.selectTerm);
    terms.add(VIEW.whenTerm);
    terms.add(VIEW.thenTerm);
    terms.add(VIEW.headTerm);
    l.add(new Merge.CardinalityConstraint(terms, 1, NONE));

    l.add(new Merge.CardinalityConstraint(VIEW.as, 1, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.conditionExpr, 1, NONE));

    l.add(new Merge.CardinalityConstraint(VIEW.aliasClause, NONE, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.aliasName, NONE, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.tableRef, NONE, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.fieldRef, NONE, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.headTerm, NONE, 1));
    l.add(new Merge.CardinalityConstraint(VIEW.operatorName, NONE, 1));

    return l;
  }


  public UntypedGateway defaultSerializer() throws ModelException {

    return new Obj2SQLView();
  }

  public Set all(Model m) throws ModelException {

//     Set cls = new HashSet();
//     cls.add(SQLViewVocabulary.ConditionExpr);
//     cls.add(SQLViewVocabulary.AliasClause);
//     cls.add(SQLViewVocabulary.Select);
//     cls.add(SQLViewVocabulary.Term);
//     cls.add(SQLViewVocabulary.CaseTerm);
//     cls.add(SQLViewVocabulary.FromClause);

    Set cls = SQLViewVocabulary.ALL;

    Set result = new HashSet();
    for(Enumeration en = m.elements(); en.hasMoreElements();) {
      Statement s = (Statement)en.nextElement();
      RDFNode o = s.object();
      if(s.predicate().equals(RDF.type) &&
	 cls.contains(s.object()))
	result.add(s.subject());
//       if(s.object() instanceof Resource && !SQLViewVocabulary.ALL.contains(s.object()))
// 	result.add(s.object());
    }
    return result;
  }

  public List getDependencies(Model m) throws ModelException {

    return new DependenciesView().getDependencies(m);
  }

  public Model select(Model m, Set nodes) throws ModelException {

    return new Subgraph().getSubgraph(m, nodes);
  }

}
