package com.interdataworking.gateway.sqlview;

import java.io.*;
import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;
import org.w3c.rdf.util.*;
import edu.stanford.db.rdf.query.*;
import edu.stanford.db.rdf.query.memory.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import edu.stanford.db.rdf.vocabulary.order_20000527.RDFX;
import org.w3c.rdf.vocabulary.dublin_core_19990702.DC;
import edu.stanford.db.rdf.query.orderings.*;
import com.interdataworking.mm.gui.TLineWriter;
import com.interdataworking.*;
import com.interdataworking.gateway.sqlddl.SQLDDLVocabulary;
import edu.stanford.db.rdf.util.*;

public class Obj2SQLView implements RDFSerializer, UntypedGateway {

  TLineWriter w;
  PowerModel model;

  static final String TAB = "    ";

  SQLDDLVocabulary sqlddl = SQLDDLVocabulary.create();

  static Set OP = new HashSet();
  static {
    OP.add("AND");
    OP.add("OR");
    OP.add("=");
    OP.add("+");
    OP.add("-");
    OP.add("%");
    OP.add("*");
    OP.add("/");
  }

  public void serialize(Model model, Writer out) throws SerializationException, IOException, ModelException {

    //    System.err.println("Num: " + model.find(null, RDF.type, R.TableDef).size());

    this.w = new TLineWriter(out);
    this.model = PowerModel.create(model);

    NodeIterator it = new NodeResultSet(Text2Ref.Q_VIEW, model);
    w.write("# View definitions\n");
    while(it.next()) {
      w.write("\nCREATE ");
      w.write("VIEW", it.get("V"));
      w.write("\n" + TAB);
      w.setTab(0);

      RDFNode table = it.get("T");
      if(table instanceof Resource)
	w.write(getTableName(table), it.get("CVT"));
      else
	w.write(getTableName(table));

      w.write("(\n");
      w.writeTab(0);
      w.write(TAB);
      w.setTab(0);

      // write target fields

      NodeIterator itc = new NodeResultSet(Text2Ref.Q_VIEW_FIELDS_ONLY_ORDERED, model);
      itc.set("V", it.get("V"));

      boolean first_itc = true;
      while(itc.next()) {

	if(!first_itc) {
	  w.write(",\n");
          w.writeTab(0);
	}
	first_itc = false;
	
	RDFNode field = itc.get("F");
	if(field instanceof Resource)
	  w.write(getFieldName(field), itc.get("CVF"));
	else
	  w.write(getFieldName(field));
      }
      w.writeTab(0, -1);
      w.write(")\nAS ");

      w.setTab(0);
      writeSelect((Resource)it.get("S"), 0, false);
      w.write('\n');
    }
    w.flush();
  }

  public void writeSelect(Resource t, int tab, boolean nested) throws IOException, ModelException {

    w.writeTab(tab);

    if(t == null) {
      w.write("NULL");
      return;
    }

    if(nested)
      w.write('(');
    w.write("SELECT", t);
    String s = model.objectS(t, SQLViewVocabulary.selectModifier);
    if(s != null)
      w.write(" " + s);
    w.write('\n');
    w.writeTab(tab, +1);
    w.write(TAB);
    w.setTab(tab+1);

    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.selectTerm);

    boolean first = true;
    while(it.next()) {

      if(!first) {
	w.write(",\n");
	w.writeTab(tab+1);
      }
      first = false;

      writeTerm((Resource)it.get("O"), tab+1);
    }

    if(first) {
      w.write("NULL");
      return;
    }

    Resource fromClause = model.objectR(t, SQLViewVocabulary.fromClause);
    if(fromClause != null)
      writeFrom(fromClause, tab);

    Resource whereClause = model.objectR(t, SQLViewVocabulary.whereClause);
    if(whereClause != null)
      writeWhere(whereClause, tab);

    if(nested)
      w.write(')');
  }

  public void writeFrom(Resource t, int tab) throws IOException, ModelException {

    if(model.count(t, SQLViewVocabulary.inFromClause, null) == 0)
      return; // empty/invalid FROM clause

    w.write('\n');
    w.writeTab(tab);
    w.write("FROM", t);
    w.write('\n');
    w.writeTab(tab);
    w.write(TAB);
    w.setTab(tab+1);
    
    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.inFromClause);

    boolean first = true;
    while(it.next()) {

      if(!first) {
	w.write(",\n");
	w.writeTab(tab+1);
      }
      first = false;

      writeAliasClause((Resource)it.get("O"), tab+1);
    }
  }

  // may be literal or resource
  public String getTableName(RDFNode t) throws ModelException {

    if(t instanceof Resource) {
      String resolvedName = RDFUtil.getObjectLiteral(model, (Resource)t, sqlddl.name);
      if(resolvedName != null)
	return resolvedName;
    }
    return String.valueOf(t);
  }

  // may be literal or resource
  public String getFieldName(RDFNode t) throws ModelException {

    return getTableName(t);
  }

  public void writeAliasClause(Resource t, int tab) throws IOException, ModelException {

    String tableName = getTableName(RDFUtil.getObject(model, t, SQLViewVocabulary.tableRef));
    Resource term = RDFUtil.getObjectResource(model, t, SQLViewVocabulary.term);
    String aliasName = RDFUtil.getObjectLiteral(model, t, SQLViewVocabulary.aliasName);

    if(term == null) {
      w.writeTab(tab);
      w.write(tableName);
    } else
      writeTerm(term, tab);

    if(aliasName != null) {
      w.write(" ");
      w.write("AS", t);
      w.write(" ");
      w.write(aliasName);
    }
  }

  public void writeWhere(Resource t, int tab) throws IOException, ModelException {

    if(model.count(t, SQLViewVocabulary.term, null) == 0)
      return; // empty/invalid WHERE clause

    w.write('\n');
    w.writeTab(tab);
    w.write("WHERE", t);
    w.write('\n');
    w.write(TAB);
    w.setTab(tab+1);
    
    Resource term = RDFUtil.getObjectResource(model, t, SQLViewVocabulary.term);

    writeTerm(term, tab+1);
  }

  public static int getPropertyCount(Model m, Resource s, Resource p) throws ModelException {

    return m.find(s, p, null).size();
  }

  // functional term, field term, constant, or select clause
  public void writeFunctionalTerm(Resource t, int tab) throws IOException, ModelException {

    String operatorName = RDFUtil.getObjectLiteral(model, t, SQLViewVocabulary.operatorName);
    String separator = ",\n";

    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.term);

    w.write(operatorName, t);

    w.write('(');
    w.setTab(tab+1);

    boolean first = true;
    while(it.next()) {

      if(!first)
	w.write(separator);
      first = false;
	
      writeTerm((Resource)it.get("O"), tab+1);
    }
    w.write(')');
  }


  // functional term, field term, constant, or select clause
  public void writeExpression(Resource t, int tab) throws IOException, ModelException {

    String operatorName = RDFUtil.getObjectLiteral(model, t, SQLViewVocabulary.operatorName);
    String separator = " " + operatorName + "\n";

    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.term);

    w.write('(');
    w.setTab(tab+1);

    boolean first = true;
    while(it.next()) {

      if(!first) {
	w.write(" ");
	if(t != null) { // create TLine box only for the first occurrence
	  w.write(operatorName, t);
	  t = null;
	} else
	  w.write(operatorName);
	w.write("\n");
      }
      first = false;
	
      writeTerm((Resource)it.get("O"), tab+1);

    }
    w.write(')');
  }

  public void writeCaseTerm(Resource t, int tab) throws IOException, ModelException {

    w.setTab(tab+1, TAB.length()); // for when/then
    w.write("CASE", t);
    w.write(" ");
    w.setTab(tab+2);
    Resource headTerm = RDFUtil.getObjectResource(model, t, SQLViewVocabulary.headTerm);
    writeTerm(headTerm, tab+2);

    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.conditionExpr);

    while(it.next()) {

      w.write('\n');
      w.writeTab(tab+1);
      Resource condExpr = (Resource)it.get("O");
      Resource whenTerm = RDFUtil.getObjectResource(model, condExpr, SQLViewVocabulary.whenTerm);
      Resource thenTerm = RDFUtil.getObjectResource(model, condExpr, SQLViewVocabulary.thenTerm);
      if(whenTerm != null) {
	w.write("WHEN ");
	w.setTab(tab+2);
	writeTerm(whenTerm, tab+2);
      }
      if(whenTerm != null && thenTerm != null) {
	w.write(" THEN ");
	w.setTab(tab+2);
	writeTerm(thenTerm, tab+2);
      }
      if(whenTerm == null && thenTerm != null) {
	w.write("ELSE ");
	w.setTab(tab+2);
	writeTerm(thenTerm, tab+2);
      }
    }
    w.write('\n');
    w.writeTab(tab);
    w.write(" END");
  }

  public void writeInExpression(Resource t, int tab) throws IOException, ModelException {

    NodeIterator it = model.getOrderedNI(t, SQLViewVocabulary.term);
    String separator = ",\n";

    it.next();
    writeTerm((Resource)it.get("O"), tab);

    w.write(" ");
    w.write("IN", t);
    w.write("(");
    w.setTab(tab+1);

    boolean first = true;
    while(it.next()) {

      if(!first)
	w.write(separator);
      first = false;
	
      writeTerm((Resource)it.get("O"), tab+1);
    }
    w.write(')');
  }

  // functional term, field term, constant, or select clause
  public void writeTerm(Resource t, int tab) throws IOException, ModelException {

    w.writeTab(tab);
    if(RDFUtil.isInstanceOf(model, t, SQLViewVocabulary.FunctionalTerm)) {

      String operatorName = RDFUtil.getObjectLiteral(model, t, SQLViewVocabulary.operatorName);

      if(operatorName.equals("IN"))
	writeInExpression(t, tab);
      else if(OP.contains(operatorName) && getPropertyCount(model, t, SQLViewVocabulary.term) > 1)
	writeExpression(t, tab);
      else
	writeFunctionalTerm(t, tab);

    } else if(RDFUtil.isInstanceOf(model, t, SQLViewVocabulary.Constant)) {

      String value = RDFUtil.getObjectLiteral(model, t, SQLViewVocabulary.constantValue);
      w.write(value, t);

    } else if(RDFUtil.isInstanceOf(model, t, SQLViewVocabulary.FieldTerm)) {

      RDFNode aliasClause = RDFUtil.getObject(model, t, SQLViewVocabulary.aliasClause);
      String fieldRef = getFieldName(RDFUtil.getObject(model, t, SQLViewVocabulary.fieldRef));
      String aliasName = null;
      if(aliasClause instanceof Resource)
	aliasName = RDFUtil.getObjectLiteral(model, (Resource)aliasClause, SQLViewVocabulary.aliasName);
      else if(aliasClause instanceof Literal)
	aliasName = aliasClause.getLabel();

      if(aliasName != null)
	w.write(aliasName + "." + fieldRef, t);
      else
	w.write(fieldRef, t);

    } else if(RDFUtil.isInstanceOf(model, t, SQLViewVocabulary.CaseTerm)) {

      writeCaseTerm(t, tab);

    } else if(RDFUtil.isInstanceOf(model, t, SQLViewVocabulary.Select)) {

      //      w.write(TAB);
      //      w.setTab(tab+1);
      writeSelect(t, tab, true);
    }
  }

  /**
   * Transforms an ordered list of models into another list of models
   */
  public List execute(List input) throws ModelException {

    Model m = (Model)input.get(0);
    StringWriter w = new StringWriter();
    try {
      serialize(m, w);
    } catch (Exception any) {}

    List result = new ArrayList();
    result.add(w.toString());
    result.add(this.w.getLines());

    //    System.err.println("Returning string:\n " + w + ", lines:\n" + lines);

    return result;
  }

  public int getMinInputLen() { return 1; }

  public int getMaxInputLen() { return 1; }

  public int getMinOutputLen() { return 2; }

  public int getMaxOutputLen() { return 2; }

}
