package com.interdataworking.gateway.xml;

import com.interdataworking.*;
import com.interdataworking.mm.*;
import com.interdataworking.mm.alg.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.util.*;
import java.util.*;
import edu.stanford.db.rdf.query.*;
import edu.stanford.db.rdf.query.memory.*;
//import edu.stanford.db.rdf.query.orderings.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;

public class CanonicalMapXSD implements UntypedGateway {

  DependenciesXSD dx = new DependenciesXSD();

  // returns list of MapPairs
  public List getMap(Model model) throws ModelException {

    List result = new ArrayList();

    //    NodeFactory f = model.getNodeFactory();

    Resource r = dx.getRoot(model);

    addChildren(r, model, result, 0, "");

    return result;
  }

  String schemaName(int i) {

    if(i == 1)
      return "SCHEMA";
    else
      return "SCHEMA" + i;
  }

  void addChildren(Resource r, Model model, Collection result, int schemaID, String prefix) throws ModelException {

    String name = null;

    if(dx.isSchema(model, r)) {
      schemaID++; // new schema encountered
      MapPair p = new MapPair(r, model.getNodeFactory().createResource(schemaName(schemaID)), 1.0);
      result.add(p);
    }

    else if(dx.isElement(model, r)) {

      Literal tn = dx.getElementNameOrRef(model, r);
      name = (tn != null ? tn.getLabel() : "NN");
      if(dx.isType(model, r) && tn != null) // this is a global name
	prefix = "";
    }

    if(name != null) {

      //      if("".equals(prefix))
      //	prefix = "S" + schemaID;

      prefix += (!"".equals(prefix) ? "." : "") + name;

      MapPair p = new MapPair(r, model.getNodeFactory().createResource(prefix), 1.0);
      result.add(p);
    }

    List children = dx.getChildren(model, r);

    for(Iterator it = children.iterator(); it.hasNext();) {

      Resource child = (Resource)it.next();
      addChildren(child, model, result, schemaID, prefix);
    }
  }

  public List execute(List input) throws ModelException {

    Model src = (Model)input.get(0);

    List map = getMap(src);

    ArrayList l = new ArrayList();
    l.add(MapPair.asModel(src.create(), map));
    return l;
  }
  

  public int getMinInputLen() { return 1; }

  public int getMaxInputLen() { return 1; }

  public int getMinOutputLen() { return 1; }

  public int getMaxOutputLen() { return 1; }
  
}
