package com.interdataworking.gateway.xml;

import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;
import org.w3c.rdf.util.*;
import edu.stanford.db.rdf.syntax.generic.*;
import edu.stanford.db.xml.util.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import java.util.*;
import java.io.*;
import java.net.*;

public class XML2RDFNC extends GenericXML2RDF {

  static final QName XML_SPACE = new QName("http://www.w3.org/XML/1998/namespace", "space");

  XMLVocabulary R;

  class XMLElement extends Element {

    Resource nodeID;
    int childNum;
    //    int posInParent;

    public XMLElement() {
      super();
    }
  }

  XMLElement root = new XMLElement();

  public XML2RDFNC() {
    super();
  }

  protected Element createElement() {
    return new XMLElement();
  }

  // implements ContentHandler
  public void startDocument() throws SAXException {

    super.startDocument();
    try {
      root.nodeID = nodeFactory.createUniqueResource();
      current = root;

      R = new XMLVocabulary(); //nodeFactory);

    } catch (Exception any) {
      throw new SAXException(any);
    }
  }

  public void endDocument() throws SAXException {

    try {
      createStatement(root.nodeID, RDF.type, R.XMLARC);
      Enumeration en = getNamespaces();
      while(en.hasMoreElements()) {
	createStatement(root.nodeID, R.usesNamespace, nodeFactory.createResource((String)en.nextElement()));
      }
    } catch (Exception any) {
      //      any.printStackTrace(System.err);
      throw new SAXException(any);
    }
    super.endDocument();
  }

  public void startElement(String uri, String local, String qname, Attributes attr) throws SAXException {

    _startElement(uri, local, attr);
    // System.out.println("Started: " + current.getName());

    try {
      XMLElement xe = (XMLElement)current;
      xe.nodeID = nodeFactory.createUniqueResource();
      XMLElement parent = (XMLElement)current.getParent();
      parent.childNum++;
      //      xe.posInParent = parent.childNum;

      for(Iterator it = xe.getAttributes().entrySet().iterator(); it.hasNext();) {

	Map.Entry e = (Map.Entry)it.next();
	QName name = (QName)e.getKey();
	String value = (String)e.getValue();

//       for(Enumeration en = xe.getAttributes(); en.hasMoreElements();) {
	
//       	QName name = (QName)en.nextElement();
	if(XML_SPACE.equals(name))
	  continue; // ignore xml:space directive

	//	String value = current.getAttribute(name);
	
	createStatement(xe.nodeID,
			nodeFactory.createResource(name.getNamespace(), name.getLocalName()),
			nodeFactory.createLiteral(value));
      }
    } catch (Exception any) {
      throw new SAXException(any);
    }
  }

  public void endElement(String uri, String local, String qname)  throws SAXException {

    try {

      XMLElement xe = (XMLElement)current;
      XMLElement parent = (XMLElement)current.getParent();

      //      System.err.println("ENDED: " + xe.getName() + " xe.childNum=" + xe.childNum);

      if(xe.childNum > 0) {

	Statement st = createStatement(parent.nodeID,
				       nodeFactory.createResource(xe.getName().getNamespace(), xe.getName().getLocalName()),
				       xe.nodeID);

	//	createStatement(st, R.ORDER, nodeFactory.createLiteral(xe.getNumInParent()));

	if((xe.getType() & xe.CDATA) != 0)
	  createStatement(xe.nodeID,
			  R.cdata,
			  nodeFactory.createLiteral(xe.getValue()));

      } else if((xe.getType() & xe.CDATA) != 0) {

	// no children, just CDATA
	Statement st = createStatement(parent.nodeID,
				       nodeFactory.createResource(xe.getName().getNamespace(), xe.getName().getLocalName()),
				       nodeFactory.createLiteral(xe.getValue()));
	//	createStatement(st, R.ORDER, nodeFactory.createLiteral(xe.getNumInParent()));

      }

      //      System.out.println("Ended: " + current.getName());

    } catch (Exception any) {
      throw new SAXException(any);
    }

    _endElement(uri, local);
  }

  public static void main(String[] args) throws Exception {

    _main(args[0], new XML2RDFNC());
  }
}
