package com.interdataworking.mm.alg;

import java.util.*;
import java.io.*;
import org.w3c.rdf.model.*;
//import java.text.*;
import com.interdataworking.*;
import com.interdataworking.mm.MapVocabulary;
//import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import org.w3c.rdf.util.*;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import edu.stanford.db.rdf.util.*;
import edu.stanford.db.rdf.model.i.*;

public class MatchSQL implements UntypedGateway {

  public int getMinInputLen() { return 2; }

  public int getMaxInputLen() { return 3; }

  public int getMinOutputLen() { return 1; }

  public int getMaxOutputLen() { return 1; }


  public void store(java.sql.Statement st, Model m, String tname) throws SQLException, ModelException {

    System.err.println("Populating database " + tname + " with " + m.size() + " entries...");

    for(Enumeration en = m.elements(); en.hasMoreElements(); ) {

      Statement s = (Statement)en.nextElement();
      if(s.subject() instanceof Statement ||
	 s.object() instanceof Statement)
	continue;

      long subj = ((LongIDAware)s.subject()).getLongID();
      long pred = ((LongIDAware)s.predicate()).getLongID();
      long obj = ((LongIDAware)s.object()).getLongID();
	
      st.execute("INSERT INTO " + tname + " VALUES " +
		 "(" + subj + "," + pred + "," + obj + ")");
    }
  }

  public List execute(List input) throws ModelException {

    Model m1 = (Model)input.get(0);
    Model m2 = (Model)input.get(1);
    List sigma0 = null;
    
    if(input.size() > 2) {
      Model initialMap = (Model)input.get(2);
      sigma0 = MapPair.toMapPairs(initialMap);
    }

    List finalList = new ArrayList(); //table.values());



    try {

      SimFloodSQL sfa = new SimFloodSQL();
      
      java.sql.Statement st = sfa.createStatement();
      
      String src = sfa.tableName("src");
      String dest = sfa.tableName("dest");

      sfa.createTable(src, sfa.TBL_SPO);
      sfa.createTable(dest, sfa.TBL_SPO);

      // populate both tables with data

      store(st, m1, src);
      sfa.createSPOIndex(src);

      store(st, m2, dest);
      sfa.createSPOIndex(dest);

      // populate default map if any

      Registry r = Registry.getDefaultRegistry();

      if(sigma0 != null) {

	System.err.println("Transferring default map of size " + sigma0.size());

	Iterator it = sigma0.iterator();
	while(it.hasNext()) {
	  
	  MapPair p = (MapPair)it.next();
	  long l1 = ((LongIDAware)p.getLeft()).getLongID();
	  long l2 = ((LongIDAware)p.getRight()).getLongID();
	  //	  System.err.println("" + l1 + " to " + l2 + ": " + p.sim);
	  //	System.err.println("l1 is " + r.get(l1));
	st.execute("INSERT INTO " + sfa.getMapPairsTableName() + " (node_a,node_b,sim) VALUES " +
		   "(" + l1 + "," + l2 + "," + p.sim + ")");
	}
      } else {
	// no default map
	sfa.defaultMapPairs(src, dest);
      }

      sfa.mjoin(src, dest, 10);
      ResultSet rs = sfa.getResult();
      while(rs.next()) {

	long n1 = rs.getLong(1);
	long n2 = rs.getLong(2);
	double sim = rs.getDouble(3);
	MapPair p = new MapPair(r.get(n1), r.get(n2), sim);
	finalList.add(p);
	//	System.err.println("" + p);
      }

      //      sfa.cleanUp();

    } catch (SQLException any) {
      System.err.println("SQL Exception: " + any);
      System.exit(1);
    }

    //    Map table = getMatch(m1, m2, sigma0);

//     Object[] a = finalList.toArray();
//     MapPair.sort(a);
//     finalList = Arrays.asList(a);

//     finalList = MapPair.prune(finalList, Match.MIN_CHOICE_ITEMS);

    Model map = MapPair.asModel(m1.create(), finalList.toArray());
    ArrayList l = new ArrayList();
    l.add(map);
    return l;
  }


}
