package com.interdataworking.mm.alg;

import com.interdataworking.*;
import com.interdataworking.mm.*;
import com.interdataworking.mm.alg.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.util.*;
import java.util.*;

/**
 * Given a set of forward and backward arcs, generates a map of
 * <obj, dependent obj>
 **/

public class Subgraph implements UntypedGateway {

  public static Model getSubgraph(Model src, Set nodes) throws ModelException {

    Model result = src.create();

    for(Enumeration en = src.elements(); en.hasMoreElements();) {

      Statement st = (Statement)en.nextElement();

      if(nodes.contains(st.subject()) &&
	 (nodes.contains(st.object()) || st.object() instanceof Literal)) {

	result.add(st);

	// try to add directly referenced reified statements for this one
	SetOperations.unite(result, src.find(st, null, null));
	SetOperations.unite(result, src.find(null, null, st));
      }
    }

    return result;
  }

  public List execute(List input) throws ModelException {

    Model src = (Model)input.get(0);
    Model selectorM = (Model)input.get(1);

    MapVocabulary M = new MapVocabulary(src.getNodeFactory());
    
    Model result = getSubgraph(src, M.asSet(selectorM));

    ArrayList l = new ArrayList();
    l.add(result);
    return l;
  }
  

  public int getMinInputLen() { return 2; }

  public int getMaxInputLen() { return 2; }

  public int getMinOutputLen() { return 1; }

  public int getMaxOutputLen() { return 1; }
  
}
