package com.interdataworking.mm.alg;

import com.interdataworking.*;
import com.interdataworking.mm.*;
//import com.interdataworking.mm.alg.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.util.*;
import java.util.*;

public class TransitiveClosure implements UntypedGateway {

  /**
   * returns either a Set or the same collection
   */
  public Collection closure(Collection map) throws ModelException {

    Map l2r = MapPair.toJavaSetMap(map, false);

    Set result;
    boolean added;

    do {
      //      System.err.println("----- looping over " + map.size() + " entries");
      added = false;
      result = new HashSet();
      result.addAll(map);
      
      for(Iterator it = map.iterator(); it.hasNext();) {
	
	MapPair p = (MapPair)it.next();
	Set s = (Set)l2r.get(p.getRight());

	//	System.err.println(" ----- considering " + p + " with target " + t);

	if(s == null)
	  continue;

	for(Iterator is = s.iterator(); is.hasNext();) {

	  MapPair t = (MapPair)is.next();

	  if(p.getLeft().equals(t.getRight()))
	    continue; // do not add reflexive

	  MapPair n = new MapPair(p.getLeft(), t.getRight());

	  n.sim = p.sim * t.sim; // just in case someone needs it
	  n.inverse = p.inverse ^ t.inverse;

	  if(result.add(n)) {
	    added = true;
	    //	    System.err.println("------- adding: " + n);
	  }
	}
      }

      map = result;

    } while(added);

    return map;
  }

  public List execute(List input) throws ModelException {

    Model mapM = (Model)input.get(0);
    List map = MapPair.toMapPairs(mapM);

    Model result = MapPair.asModel(mapM, closure(map));

    ArrayList l = new ArrayList();
    l.add(result);
    return l;
  }
  

  public int getMinInputLen() { return 1; }

  public int getMaxInputLen() { return 1; }

  public int getMinOutputLen() { return 1; }

  public int getMaxOutputLen() { return 1; }
  
}
