package com.interdataworking.mm.gui;

import javax.swing.*;
import javax.swing.tree.*;
//import javax.swing.event.*;
//import javax.swing.text.*;
//import javax.swing.border.*;
//import javax.swing.colorchooser.*;
//import javax.swing.filechooser.*;
import javax.swing.table.*;
import javax.swing.event.ChangeEvent;
import javax.accessibility.*;

//import java.awt.*;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Image;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.event.*;
import java.awt.geom.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import java.applet.*;
import java.net.*;
import java.text.DecimalFormat;

public class ImageComponent extends JComponent {

  static final int PADDING = 2; // pixel

  Image img;
  String text;
  Map lines; // RDFNode -> TLine
  //  Collection boxes = new ArrayList();
  BoxUpdater boxUpdater;

  public ImageComponent(String text, Map lines, BoxUpdater boxUpdater) {

    this.boxUpdater = boxUpdater;
    this.lines = lines;
    this.text = text;
    recalcBounds(1, 1);
  }

  public ImageComponent(Image img) {

    this.img = img;
    recalcBounds(img.getWidth(null), img.getHeight(null));
  }

//   public Collection getBoxes() {

//     return boxes;
//   }

  void recalcBounds(int w, int h) {

    if(w < 0)
      w = 1;
    if(h < 0)
      h = 1; // esp. for empty text boxes
    Dimension d = new Dimension(w, h);
    setPreferredSize(d);
    setMinimumSize(d);
    setMaximumSize(d);
    setBounds(new Rectangle(0, 0, w, h));
  }

  int getHeight(FontMetrics fm) {

    return fm.getHeight() + PADDING;
  }

  public void paint(Graphics g) {

    if(img == null) {

      int n = lineNum(text);
      FontMetrics fm = g.getFontMetrics();

      img = createImage(Math.max(textWidth(text, fm), 1), Math.max(getHeight(fm) * n, 1));
      //      System.err.println("Created image: " + img);
      Graphics offgraphics = img.getGraphics();
      offgraphics.setColor(Color.white);
      offgraphics.fillRect(0, 0, img.getWidth(null), img.getHeight(null));
      offgraphics.setColor(Color.black);
      drawText(text, fm, offgraphics);
      recalcBounds(img.getWidth(null), img.getHeight(null));
      Map boxes = getBoxes(text, fm, lines);
      boxUpdater.updateBoxes(boxes, this);
    }

    //    System.err.println("Drawing image");
    g.drawImage(img, 0, 0, null);
  }

  public static int lineNum(String s) {

    BufferedReader in = new BufferedReader(new StringReader(s));

    String l;
    int c = 0;

    try {
      while((l = in.readLine()) != null)
	c++;
    } catch (IOException any) {}

    return c;
  }

  public static int textWidth(String s, FontMetrics fm) {

    BufferedReader in = new BufferedReader(new StringReader(s));

    String l;
    int c = 0;

    try {
      while((l = in.readLine()) != null)
	c = Math.max(c, fm.stringWidth(l));
    } catch (IOException any) {}

    return c;
  }

  // returns rectangles
  public void drawText(String s, FontMetrics fm, Graphics g) {

    BufferedReader in = new BufferedReader(new StringReader(s));

    String l;
    int c = getHeight(fm);

    try {
      while((l = in.readLine()) != null) {
	g.drawString(l, 0, c);
	c += getHeight(fm);
      }
    } catch (IOException any) {}
  }

  // returns rectangles
  public Map getBoxes(String s, FontMetrics fm, Map tlines) {

    Map boxes = new HashMap(); // RDFNode -> JComponent (PointerBox)

    BufferedReader in = new BufferedReader(new StringReader(s));

    String l;
    int c = getHeight(fm);
    int row = 0;

    try {
      while((l = in.readLine()) != null) {
	row++;
	// check lines: FIXME efficiency
	for(Iterator it = tlines.entrySet().iterator(); it.hasNext();) {

	  Map.Entry e = (Map.Entry)it.next();

	  Object peer = e.getKey();
	  TLine t = (TLine)e.getValue();

	  if(t.row == row) {

	    try {
	      PointerBox box = new PointerBox(fm.stringWidth(l.substring(0, t.start)) - PADDING,
					      c - getHeight(fm) + PADDING,
					      fm.stringWidth(l.substring(t.start, t.end)) + PADDING*2,
					      getHeight(fm));
	      boxes.put(peer, box);
	    } catch (Exception any) {
	      System.err.println("[ImageComponent] could not create box for " + t + " at " + l);
	    }
	  }
	}
	c += getHeight(fm);
      }
    } catch (IOException any) {}

    return boxes;
  }

}
