package edu.stanford.db.alg.ngram;

import java.util.*;

/**
 * Given two lists of objects, returns the textual similarities of
 * the objects in both lists based on n-gram matching.
 */

public class NGramMatcher {

//   public NGramMatcher(List l1, List l2) {
//   }

  static public ElementSimilarity[] joinSort(Collection c1, Collection c2, int minLen, int maxLen) {

    ElementSimilarity[] similarities = join(c1, c2, minLen, maxLen);

    // dump similarities
    ElementSimilarity.sort(similarities);

    for(int i=0; i < similarities.length; i++) {

      System.err.println("" + similarities[i]);
    }

    return similarities;
  }

  /**
   * returns an unsorted array of ElementSimilarities
   */
  static public ElementSimilarity[] join(Collection c1, Collection c2, int minLen, int maxLen) {

    // create outer sorted array
    Object[] outer = intoNGramArray(invert(c1, minLen, maxLen));
    Arrays.sort(outer, Entry.getLengthComparator());

    System.err.println("Inverted collection 1.");

    // create inner map
    Object[] ngrams2 = intoNGramArray(invert(c2, minLen, maxLen));
    Map inner = new HashMap(); // self -> self of NGrams

    for(int i = 0; i < ngrams2.length; i++)
      inner.put(ngrams2[i], ngrams2[i]);

    System.err.println("Inverted collection 2.");

    // join

    System.err.println("Joining inverted lists " + outer.length + " x " + inner.size());

    Map sims = new HashMap(); // self, self of Similarities

    for(int i=0; i < outer.length; i++) {

      NGram ngramOuter = (NGram)outer[i];
      NGram ngramInner = (NGram)inner.get(ngramOuter);

//       System.err.println("Outer: " + ngramOuter);
//       System.err.println("Inner: " + ngramInner);

      if(ngramInner == null)
	continue; // nothing to join

      // merge-join both sorted lists of Entries
      mergeJoin(ngramInner, ngramOuter, sims);
    }

    
    ElementSimilarity[] similarities = toSimilarityArray(sims.values());

    // normalize to 1.0
    double max = 0.000001;
    for(int i=0; i < similarities.length; i++) {
      if(similarities[i].getSim() > max)
	max = similarities[i].getSim();
    }
    max = max * 1.01;
    for(int i=0; i < similarities.length; i++) {
      similarities[i].setSim(similarities[i].getSim()/max);
      //      System.err.println("El of type: " + similarities[i].getElement1().getClass());
    }

    // discovered similar pairs w/o enumerating cross product.
    // now, compute accurate similarity values

    computeAccurateValues(similarities);

    System.err.println("Joined collections: " + similarities.length + " matches");

    inner = null;
    outer = null;
    System.gc();

    return similarities;
  }

  static void computeAccurateValues(ElementSimilarity[] sims) {

    for(int i=0; i < sims.length; i++) {

      if(sims[i].getElement1().equals(sims[i].getElement2()))
	sims[i].setSim(1.0);
    }
  }

  public static ElementSimilarity[] toSimilarityArray(Collection c) {

    ElementSimilarity[] a = new ElementSimilarity[c.size()];

    Iterator it = c.iterator();

    for(int i = 0; i < c.size(); i++)
      a[i] = (ElementSimilarity)it.next();

    return a;
  }

  static protected double getMultiplierForNGram(int length) {

//     switch(length) {

//     case 1: return 0.01;
//     case 2: return 0.1;
//     case 3: return 1.0;
//     }

    return length * length * length * length;

    //    return 1.0;
  }

  static protected void mergeJoin(NGram ngramInner, NGram ngramOuter, Map sims) {

    Object[] entriesInner = ngramInner.getEntries();
    Object[] entriesOuter = ngramOuter.getEntries();

    String ngram = (String)ngramInner.getElement();

    //    System.err.println("MERGING " + ngram + ": " + entriesInner.length + " x " + entriesOuter.length);

//     dumpInvlist(entriesInner);
//     System.err.println(" and");
//     dumpInvlist(entriesOuter);

    ElementSimilarity container = ElementSimilarity.createContainer();

    for(int i = 0; i < entriesInner.length; i++) {

      Entry inner = (Entry)entriesInner[i];

      for(int j = 0; j < entriesOuter.length; j++) {

	Entry outer = (Entry)entriesOuter[j];

	ElementSimilarity so = ElementSimilarity.get(sims, outer.getElement(), inner.getElement(), container);

	//	double localMult = inner.getCount() * outer.getCount();
	double termFreq = (inner.getCount() + outer.getCount()) / 2;
	double docFreq = ngramInner.getCount() * ngramInner.getCount();
	// double docFreq = (ngramInner.getCount() + ngramInner.getCount()) / 2;
	// double docFreq = 1.0;

	double termLen = (inner.getElement().toString().length() * outer.getElement().toString().length());
	// double termLen = (inner.getElement().toString().length() + outer.getElement().toString().length()) / 2;

	double increment = getMultiplierForNGram(ngram.length()) * termFreq / docFreq / termLen;

	//	System.err.println("  inc for " + inner + " and " + outer + " = " + increment);

	so.incSim(increment);
      }
    }
  }

  /**
   * returns Map: string -> collection of entries
   */
  static protected Map invert(Collection obj, int minLen, int maxLen) {

    Map invlist = new HashMap(); // maps string -> collection of Entries

    Iterator it = obj.iterator();
    while(it.hasNext()) {

      Object el = it.next();

      Map ngrams = chopIntoNGrams(el, minLen, maxLen);

      // translate the entries and add them appropriately:
      // (ngram, count) -> (objID, count)

      Iterator it_e = ngrams.values().iterator();

      while(it_e.hasNext()) {

	Entry ngramEntry = (Entry)it_e.next();
	Object ngramStr = ngramEntry.getElement();

	// fetch collection of this ngram
	Collection c = (Collection)invlist.get(ngramStr); // lookup by string
	if(c == null) {
	  c = new ArrayList();
	  invlist.put(ngramStr, c);
	}
	//	Entry docEntry = new Entry(el, ngramEntry.getCount());
	// reuse same entry
	Entry docEntry = ngramEntry;
	docEntry.setElement(el); // count remains the same
	c.add(docEntry);
      }

    }

    return invlist;
  }


  /**
   * returns unsorted ngram array
   */
  static protected Object[] intoNGramArray(Map invlist) {

    // get total count of entries, replace key (string) by NGram instance

    Iterator it1 = invlist.entrySet().iterator();
    
    //    Map invlist2 = new HashMap(); // end result
    
    Object[] ngrams = new Object[invlist.size()];
    int ng = 0;
    
    while(it1.hasNext()) {

      Map.Entry e = (Map.Entry)it1.next();

      Collection c = (Collection)e.getValue();

      Iterator it2 = c.iterator();
      Object[] entries = new Object[c.size()];

      int i = 0;
      int total = 0;

      while(it2.hasNext()) {

	Entry se = (Entry)it2.next();
	total += se.getCount();
	entries[i++] = se;
      }

      Arrays.sort(entries, Entry.getElementComparator());
      
      NGram ngram = new NGram((String)e.getKey(), total, entries);

      //      invlist2.put(ngram, entries);
      ngrams[ng++] = ngram;

      //      System.err.println(res);
    }

    return ngrams;
  }

  static private void dumpInvlist(Object[] ngrams) {

    // dump invlist

    for(int i=0; i < ngrams.length; i++) {
      
      System.err.println("" + ngrams[i]);
    }
  }

  static private void dumpInvlist1(Map invlist) {

    // dump invlist

    Iterator it1 = invlist.entrySet().iterator();

    while(it1.hasNext()) {

      Map.Entry e = (Map.Entry)it1.next();

      String res = "n-gram: " + e.getKey() + ", entries: ";

      Iterator it2 = ((Collection)e.getValue()).iterator();

      while(it2.hasNext()) {

	Entry se = (Entry)it2.next();

	res += " " + se;
      }

      System.err.println(res);
    }
  }

  static protected String stringify(Object el) {

    String value = el.toString().toLowerCase();
    return value;
  }

  static protected boolean isSeparator(char c) {

    //    return !Character.isLetterOrDigit(c);
    return !Character.isLetter(c);
  }

  static protected Map chopIntoNGrams(Object el, int minLen, int maxLen) {

    String value = stringify(el);

    // collect all n-grams and their frequencies

    Map ngrams = new HashMap(); // string -> Entry

    for(int i=0; i < value.length(); i++) {

      int j = i + 1;

      char c = value.charAt(i);

      // if standing on separator, try to get the whole word
      if(i == 0 || isSeparator(c)) {

	int ws, we;
	ws = we = (i == 0 ? 0 : i + 1);

	// get the word
	while(we < value.length() && !isSeparator(value.charAt(we)))
	  we++;

	int len = we - ws;
	if(len > maxLen || (len < minLen && len > 0))
	  recordNGram(value.substring(ws, we), ngrams); // record whole word, either too small or too large

	if(len < minLen) { // skip a too small word completely
	  i = we;
	  continue;
	}
      }

      while (j <= value.length()) {

	if((j - i >= minLen && j - i <= maxLen) /*||
						  isSeparator(c)*/) {
	
	  String ngramStr = value.substring(i, j);
	  recordNGram(ngramStr, ngrams);
	}

	if((j < value.length() && isSeparator(value.charAt(j))) ||
	   (isSeparator(c)))
	  break; // include separator only as a single character

	j++;
      }
    }
    
    // debug

    /*
    System.err.println("NGrams for: " + el);

    Iterator it = ngrams.values().iterator();

    while(it.hasNext()) {

      System.err.println("" + it.next());
    }
    */

    return ngrams;
  }

  static protected void recordNGram(String ngramStr, Map ngrams) {

    //    System.err.println("Recording ngram/term: " + ngramStr);
	  
    Entry entry = (Entry)ngrams.get(ngramStr);
	  
    if(entry == null) {
      entry = new Entry(ngramStr);
      ngrams.put(ngramStr, entry);
    }
	  
    entry.incCount();
  }

  public static void main(String[] args) throws Exception {

//     final String[] c1 = { "black", "big", "bulky", "boy" };
//     final String[] c2 = { "white", "whimpy", "thin", "girl" };

    List l = Arrays.asList(args);
    List l1 = l.subList(0, args.length/2);
    List l2 = l.subList(args.length/2, args.length);

    System.err.println("List1: " + l1 + " (till " + args.length/2 + ")");
    System.err.println("List2: " + l2);

    join(l1, l2, 2, 5);

    //    invert(Arrays.asList(args));
    //    chopIntoNGrams(args[0]);
  }
}
