/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package edu.stanford.db.rdf.query;

import org.w3c.rdf.model.*;
import org.w3c.rdf.util.RDFUtil;

import java.util.*;

/**
 * A query iterator interface.
 *
 * @author Sergey Melnik <melnik@db.stanford.edu> */

public interface QueryIterator {

    // resets iterator with the preset variables. If model passed,
    // resets this and underlying iterators with its value
    public void reset(Hashtable vars, Model model) throws ModelException;

    // returns true if match found (vars updated if necessary)
    public boolean next(Hashtable vars) throws ModelException;

    // returns names of the variables used
    public Enumeration getVariables() throws ModelException;

    // returns execution cost of the iterator (e.g. stylizised number
    // of instructions/method calls)
    public int getCost();

    public QueryIterator duplicate();
}
