package edu.stanford.db.rdf.query.memory;

import org.w3c.rdf.model.*;
import edu.stanford.db.rdf.query.*;
import org.w3c.rdf.util.*;
import edu.stanford.db.rdf.vocabulary.order_20000527.RDFX;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import org.w3c.rdf.vocabulary.dublin_core_19990702.DC;
import org.w3c.rdf.vocabulary.rdf_schema_19990303.RDFS;
import edu.stanford.db.rdf.query.conditions.*;
import edu.stanford.db.rdf.query.conditions.numeric.*;
import edu.stanford.db.rdf.query.orderings.*;

import java.util.*;

/**
 * Provides an example of using iterator-based queries and conditions.
 */
public class Example {

  static RDFFactory rf = new RDFFactoryImpl();
  static NodeFactory f = rf.getNodeFactory();
  static Resource R_location = createR("location");
  static Resource R_myDevice = createR("MyDevice");
  static Resource R_room = createR("room");
  static Resource R_floor = createR("floor");
  static Resource R_admin = createR("admin");
  static Resource R_checkedOn = createR("checkedOn");


  static SubstitutionComparator[] Q_0_CHAIN = { new GroupBy("S"), new GroupBy("P"), new GroupBy("O") };
  static QueryIterator Q_0 =
    new OrderIterator(new FindIterator("S", "P", "O"),
		      new Chained(Q_0_CHAIN)
		      );


  static QueryIterator Q_1 = new FindIterator("I", RDF.type, "C");

  static QueryIterator[] Q_2 = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_admin, "A")
  };

  static QueryIterator[] Q_3 = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_location, "L"),
    new FindIterator("L", R_room, "R"),
    new OptionalIterator(new FindIterator("L", R_floor, "F"))
  };

  static QueryIterator[] Q_4_ = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_admin, "A")
  };
  static QueryIterator Q_4A = new GroupByIterator("A", new JoinIterator(Q_4_));
  static QueryIterator Q_4B = new GroupByIterator("A", "skip", new JoinIterator(Q_4_));

  static QueryIterator[] Q_5 = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_location, "L"),
    new ConditionIterator("F", new EqualNumC(createL("5")),
			  new FindIterator("L", R_floor, "F"))
  };

  static QueryIterator[] Q_6 = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_location, "L"),
    new ConditionIterator("R", new GreaterEqualC(createL("200")),
			  new FindIterator("L", R_room, "R"))
  };

  static QueryIterator[] Q_7_ = {
    new FindIterator("D", RDF.type, R_myDevice),
    new FindIterator("D", R_location, "L"),
    new FindIterator("L", R_room, "R"),
    new FindIterator("L", R_floor, "F")
  };
  static String[] Q_7_F_R = {"F", "R"};
  static QueryIterator Q_7 = new ConditionIterator(Q_7_F_R, new Less(),
						   new JoinIterator(Q_7_));

//   static QueryIterator Q_8__ = 
//     new FindIterator("S", "P", "O");
//   static QueryIterator Q_8_ = 
//     new ConditionIterator("O", new IsLiteral(),
// 			  new FindIterator("S", "P", "O"));
  static QueryIterator Q_8 =
    new OrderIterator(new ConditionIterator("O", new IsLiteral(),
					    new FindIterator("S", "P", "O")),
		      new Lexicographic("O")
		      );

  // static SubstitutionComparator[] Q_9_CHAIN = { new Numeric("O") };
  static SubstitutionComparator[] Q_9_CHAIN = { new GroupBy("S"), new Reverse(new Numeric("O")) };
  static QueryIterator Q_9 =
    new OrderIterator(new ConditionIterator("O", new IsLiteral(),
					    new FindIterator("S", "P", "O")),
		      new Chained(Q_9_CHAIN)
		      );

  static QueryIterator[] Q_10_ = {
    new FindIterator("L1", R_floor, "F1"),
    new FindIterator("L2", R_floor, "F2")
  };
  static QueryIterator Q_10 = new ConditionIterator("F1", "F2", new LessEqual(),
						    new JoinIterator(Q_10_));

  public static void print(NodeIterator it) throws ModelException {

    while(it.next()) {

      Map map = it.getAll();
      String res = "[";
      for(Iterator i = map.entrySet().iterator(); i.hasNext();) {

	Map.Entry e = (Map.Entry)i.next();
	res += "<" + e.getKey() + "=" + e.getValue() + ">";
	if(i.hasNext())
	  res += " ";
      }
      System.out.println(res + "]");
    }
  }

  static Resource createR(String name) {

    try {
      return f.createResource(name);
    } catch (ModelException any) {
    }
    return null;
  }

  static Literal createL(String name) {

    try {
      return f.createLiteral(name);
    } catch (ModelException any) {
    }
    return null;
  }

  /**
   * This is an ugly way to create a model, but it serves the purpose.
   * Effectively, the model is embedded in this Java file.
   */
  public static Model constructModel() throws ModelException {

    Model m = rf.createModel();

    m.add(f.createStatement(R_myDevice, RDF.type, RDFS.Class));

    m.add(f.createStatement(createR("device_001"), RDF.type, R_myDevice));
    m.add(f.createStatement(createR("device_001"), R_location, createR("loc_001")));
    m.add(f.createStatement(createR("device_001"), R_admin, createL("Sandip")));
    m.add(f.createStatement(createR("loc_001"), R_floor, createL("5")));
    m.add(f.createStatement(createR("loc_001"), R_room, createL("543")));

    // notice: floor is missing, but multiple admins
    m.add(f.createStatement(createR("device_002"), RDF.type, R_myDevice));
    m.add(f.createStatement(createR("device_002"), R_location, createR("loc_002")));
    m.add(f.createStatement(createR("device_002"), R_admin, createL("Sandip")));
    m.add(f.createStatement(createR("device_002"), R_admin, createL("Murugesh")));
    m.add(f.createStatement(createR("loc_002"), R_room, createL("123")));

    // notice: location speads across two floors; floor numbers are prefixed with zeroes
    //         room is not numeric
    m.add(f.createStatement(createR("device_003"), RDF.type, R_myDevice));
    m.add(f.createStatement(createR("device_003"), R_location, createR("loc_003")));
    m.add(f.createStatement(createR("device_003"), R_admin, createL("Sergey")));
    m.add(f.createStatement(createR("loc_003"), R_floor, createL("0004")));
    m.add(f.createStatement(createR("loc_003"), R_floor, createL("00005")));
    m.add(f.createStatement(createR("loc_003"), R_room, createL("Music Hall")));

    return m;
  }

  public static void main(String[] args) throws Exception {

    Model m = constructModel();
    System.out.println("=== SOURCE MODEL: grouped by subject, predicate, object");
    print(new NodeResultSet(Q_0, m));
    //    RDFUtil.printStatements(m, System.out);

    System.out.println("\n--- QUERY: all direct instances and their classes ---");
    print(new NodeResultSet(Q_1, m));

    System.out.println("\n--- QUERY: administrators of all devices ---");
    print(new NodeResultSet(Q_2, m));
    System.out.println("--- NOTICE: 'Sandip' occurs twice ---");

    System.out.println("\n--- QUERY: rooms and (optionally) floors on which all devices are located ---");
    print(new NodeResultSet(Q_3, m));
    System.out.println("--- NOTICE: floor of device_002 is missing, floor of device_003 occurs twice ---");

    System.out.println("\n--- QUERY: unique administrators of all devices ---");
    print(new NodeResultSet(Q_4A, m));

    System.out.println("\n--- QUERY: unique administrators of all devices with 'skip' variable set ---");
    print(new NodeResultSet(Q_4B, m));

    System.out.println("\n--- QUERY: all devices on 5th floor ---");
    print(new NodeResultSet(Q_5, m));
    System.out.println("--- NOTICE: 5==00005 ---");

    System.out.println("\n--- QUERY: all devices with room number greater than or equal to 200 ---");
    print(new NodeResultSet(Q_6, m));

    System.out.println("\n--- QUERY: all devices with floor number less than room number ---");
    print(new NodeResultSet(Q_7, m));
    System.out.println("--- NOTICE: floor of device_002 is missing, therefore it is not returned ---");
    System.out.println("--- NOTICE: room of device_003 is non-numeric, therefore comparison fails, too ---");

    System.out.println("\n--- QUERY: literals of all statements in lexicographic order ---");
    print(new NodeResultSet(Q_8, m));

    System.out.println("\n--- QUERY: all statements sorted first by subject and then by literals in descending numeric order ---");
    print(new NodeResultSet(Q_9, m));

    System.out.println("\n--- QUERY: all pairs of locations where floor of first is less than or equal to floor of second ---");
    print(new NodeResultSet(Q_10, m));
  }
}
