/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package edu.stanford.db.rdf.query.memory;

import org.w3c.rdf.model.*;
import edu.stanford.db.rdf.query.*;

import java.util.*;

/**
 * This is a query iterator that implements optional.
 *
 * @author Sergey Melnik <melnik@db.stanford.edu> */

public class OptionalIterator implements QueryIterator {

  boolean noMore = false;
  boolean firstPass = true;
  QueryIterator it;

  public OptionalIterator(QueryIterator it) {

    this.it = it;
  }

  // resets iterator with the preset variables. If model passed,
  // resets this and underlying iterators with its value
  public void reset(Hashtable vars, Model model) throws ModelException {

    it.reset(vars, model);
    noMore = false;
    firstPass = true;
  }

  // returns true if match found (vars updated if necessary)
  public boolean next(Hashtable vars) throws ModelException {

//     System.err.println("OPT: " + noMore + ", " + firstPass);

    if(noMore)
      return false;

    boolean b = it.next(vars);

//     System.err.println("OPT got: " + b);
//     if(b)
//       System.err.println("VARS: " + vars);

    if(!b)
      noMore = true;

    if(firstPass) {
      firstPass = false;
      return true;
    }

    return b;
  }

  // returns names of the variables used
  public Enumeration getVariables() throws ModelException {

    return it.getVariables();
  }

  // returns execution cost of the iterator (e.g. stylizised number
  // of instructions/method calls)
  public int getCost() {

    return it.getCost();
  }

  public QueryIterator duplicate() {

    OptionalIterator n = new OptionalIterator(it.duplicate());
    n.noMore = noMore;
    n.firstPass = firstPass;
    return n;
  }

}
