package edu.stanford.db.rdf.query.orderings;

import edu.stanford.db.rdf.query.*;
import org.w3c.rdf.model.*;
import java.util.*;


/**
 * Enforces a unique order on RDF nodes.
 * Don't rely on the ordering produced by this class!
 * It only makes sure that equal objects are grouped together,
 * current lexicographic order is incidental and may be changed.
 */

public class GroupBy implements SubstitutionComparator {

  String varName;

  public GroupBy(String varName) {

    this.varName = varName;
  }

  public int compare(Map s1, Map s2, Model m) throws ModelException {
    
    Object o1 = s1.get(varName);
    Object o2 = s2.get(varName);

    if(o1 == null && o2 != null)
      return -1;
    else if(o1 != null && o2 == null)
      return 1;
    else if(o1 == null && o2 == null)
      return 0;

    // both are non-null here

    return compare(o1, o2);
  }

  int compare(Object o1, Object o2) throws ModelException {
    // let literals go before resources before statements

    int lrs1 = lrs(o1);
    int lrs2 = lrs(o2);

    if(lrs1 < lrs2)
      return -1;
    else if(lrs1 > lrs2)
      return 1;

    // objects are of the same type here

    switch(lrs1) {
    case 1: // literals
      return ((Literal)o1).getLabel().compareTo(((Literal)o2).getLabel());
    case 2: { // resources
      Resource r1 = (Resource)o1;
      Resource r2 = (Resource)o2;
      int v = r1.getURI().compareTo(r2.getURI());
      if(v < 0)
	return -1;
      else if(v > 0)
	return 1;
      else
	return v;
    }
    default: // statements
      Statement s1 = (Statement)o1;
      Statement s2 = (Statement)o2;
      int v = compare(s1.subject(), s2.subject());
      if(v != 0)
	return v;
      v = compare(s1.predicate(), s2.predicate());
      if(v != 0)
	return v;
      return compare(s1.object(), s2.object());
    }
  }

  int lrs(Object o) {

    if(o instanceof Literal)
      return 1;
    if(o instanceof Statement)
      return 3;
    else
      return 2;
  }

}
