package edu.stanford.db.rdf.query.orderings;

import edu.stanford.db.rdf.query.*;
import org.w3c.rdf.model.*;
import java.util.*;


/**
 * Order by a given variable name using a <tt>java.util.Comparator</tt>
 */

public class Numeric implements SubstitutionComparator {

  String varName;

  public Numeric(String varName) {

    this.varName = varName;
  }

  public int compare(Map s1, Map s2, Model m) throws ModelException {
    
    Object o1 = s1.get(varName);
    Object o2 = s2.get(varName);

    if(o1 == null && o2 != null)
      return -1;
    else if(o1 != null && o2 == null)
      return 1;
    else if(o1 == null && o2 == null)
      return 0;

    // both are non-null here

    boolean p1 = false;
    double v1 = 0;

    try {
      v1 = Double.parseDouble(((RDFNode)o1).getLabel());
      p1 = true;
    } catch (NumberFormatException nfe) {
    }

    boolean p2 = false;
    double v2 = 0;

    try {
      v2 = Double.parseDouble(((RDFNode)o2).getLabel());
      p2 = true;
    } catch (NumberFormatException nfe) {
    }

    if(p1 && !p2)
      return -1;
    else if(!p1 && p2)
      return 1;
    else if(!p1 && !p2)
      return 0; // both are unparseable

    // both are parseable
    if(v1 < v2)
      return -1;
    else if(v2 < v1)
      return 1;

    return 0;
  }
}
