package edu.stanford.db.rdf.util;

import java.util.*;

public class TestCache {

  static void test(int cacheSize, int domain, int iterations, boolean testKey) {

    Cache cache = new Cache(cacheSize);
    // WeakHashMap cache = new WeakHashMap();

    Random rnd = new Random(150775);

    int misses = 0;

    Set pinWall = new HashSet();

    for(int i=0; i < iterations; i++) {

      long v = rnd.nextInt(domain);
      TestValue value = new TestValue(v);

      Object key = testKey ? (Object)("s" + v) : (Object)value;

      boolean notInInv = false, notInC = false;

      if(pinWall.contains(key)) {
	// must also be in the cache
 	if(cache.get(v) == null) {
	  notInInv = true;
 	}

	//	if(cache.get(new Long(v)) == null) {
	if(cache.get(key) == null) {
	  notInC = true;
	}
      }

      if((testKey &&  (notInInv || notInC) && (notInInv != notInC)) ||
	 (!testKey && (notInInv || notInC))) {

	misses++;

	if(notInInv) {
	  System.err.println("Value " + v + " is in pinwall but not in the inverse cache!");
	  System.err.println("INVERSE: " + cache.inverse);
	}

	if(notInC) {
	  System.err.println("Value " + v + " is in pinwall but not in the cache!");
	  System.err.println("MAP: " + cache.map);
	}

	//	  System.err.println("Cache size: " + cache.size());
	//	System.err.println("Cache size: " + cache.size());
	//	  System.err.println(cache.hash);
	System.err.println("PINWALL: " + pinWall);
      }

      Object cached = cache.get(key);

      if(rnd.nextBoolean()) {
	//	if(pinWall.contains(value))
	pinWall.remove(key);
	pinWall.add(cached != null ? cached : value);
	if(pinWall.size() > cacheSize)
	  pinWall.clear();
      }

      //      cache.put(new Long(v), value);
      // THIS IS THE CRITICAL BIT
      //      boolean cached = cache.get(key) != null;
//       if(overriden)
// 	System.err.println("Overriding: " + key + ", cache size: " + cache.size() + ", "); // + cache.hash.size());
      //      synchronized(cache) {
      // need to avoid WeakHashMap bug!
      //      cache.remove(key);
      if(cached == null) {
	cache.put(key, value);
	if(cache.get(key) != value)
	  System.err.println("Overriding failed: " + key);
      }
	//      }
    }
    pinWall = null;
    System.gc();
    //    System.err.println(cache);
    System.err.println("Misses: " + misses);
  }

  public static void main(String[] args) {

    int cacheSize = 1000;
    int domain = 2000;
    int iterations = 300000;

    System.err.println("\nTESTING VALUES");
    test(cacheSize, domain, iterations, false);

    System.err.println("\nTESTING KEYS");
    test(cacheSize, domain, iterations, true);
  }

  static private class TestValue implements LongIDAware {

    long id;

    public TestValue(long id) {

      this.id = id;
    }

    public long getLongID() {

      return id;
    }

    public int hashCode() {

      return (int)id;
    }

    public boolean equals(Object that) {

      if(that instanceof LongIDAware)
	return id == ((LongIDAware)that).getLongID();
      else
	throw new RuntimeException("Equals: " + that);
    }

    public String toString() {

      return "V[" + id + ", " + Integer.toHexString(System.identityHashCode(this)) + "]";
    }

    //    public void finalize() {

      //      System.err.println("Finalized: " + this);
    //    }
  }
}
