package org.w3c.rdf.examples;

import org.xml.sax.SAXException;
import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;
import org.w3c.rdf.util.*;

/**
 * A simple demo program that parses RDF at a given URL,
 * serializes it back into RDF/XML, and dumps the triples.
 *
 * <p>Try <code>java org.w3c.rdf.examples.ParseAndSerialize http://www-db.stanford.edu/~melnik/</code>
 */

public class ParseAndSerialize {

  static void bailOut() {
    System.err.println("Usage: ParseAndSerialize [-parser <parser class> | -serializer <serializer class>] <fileNameOrURL>\n" + 
		       "Some values for <parser class>: org.w3c.rdf.implementation.syntax.sirpac.SiRPAC\n" +
		       "                                edu.stanford.db.rdf.syntax.generic.TripleParser\n" +
		       "                                edu.stanford.db.rdf.syntax.strawman.StrawmanParser\n" +
		       "Some values for <serializer>:   org.w3c.rdf.implementation.syntax.sirpac.SiRS\n" + 
		       "                                edu.stanford.db.rdf.syntax.generic.TripleSerializer\n" +
		       "                                edu.stanford.db.rdf.syntax.generic.DumpConsumer\n" +
		       "If no options are specified the default parser (SiRPAC) and serializer (SiRS) are used.\n" +
		       "Use JVM option -Dorg.xml.sax.parser=<classname> to replace default XML parser.");
    System.exit(1);
  }

  public static void main(String[] args) throws Exception {

    String fileNameOrURL = null;

    String parserClass = null;
    String serializerClass = null;

    for(int i = 0; i < args.length; i++) {

      if(args[i].startsWith("-p"))
	parserClass = args[++i];
      else if(args[i].startsWith("-s"))
	serializerClass = args[++i];
      else
	fileNameOrURL = args[i];
    }

    if(fileNameOrURL == null)
      bailOut();

    RDFFactory f = new RDFFactoryImpl();
    Model m = f.createModel();

    RDFParser parser = parserClass == null ? f.createParser() : (RDFParser)Class.forName(parserClass).newInstance();
    RDFSerializer serializer = serializerClass == null ? f.createSerializer() : (RDFSerializer)Class.forName(serializerClass).newInstance();
    
    try {
      RDFUtil.parse(fileNameOrURL, parser, m);
    } catch (SAXException se) {
      System.err.println("Exception during parsing: " + se.getMessage());
      throw se.getException();
    }

//     if(printTriples) {
//       System.err.println("\nTRIPLES:\n");
//       RDFUtil.printStatements(m, System.out);
//     }

    System.err.println("\nSERIALIZED MODEL:\n");
    RDFUtil.dumpModel(m, System.out, serializer);

    System.err.println("\nModel " + m.getURI() + " of size " + m.size() + "\n");
  }
}

